from concurrent import futures
import grpc
import cv2
import Server_pb2
import Server_pb2_grpc
import time
import base64
import sys

cap = cv2.VideoCapture(0)


def Request(frame):
    ret, buf = cv2.imencode('.jpg', frame)
    if ret != 1:
        return
    data = Server_pb2.Request(datas=buf.tobytes())
    print(len(data))
    yield data


def run():
    # 送信先を指定
    channel = grpc.insecure_channel('127.0.0.1:50051')
    stub = Server_pb2_grpc.MainServerStub(channel)

    while True:

        try:
            ret, frame = cap.read()
            if ret != 1:
                continue
            cv2.imshow('Capture Image', frame)
            k = cv2.waitKey(1)
            if k == 27:
                break
            responses = stub.getStream(Request(frame))
            for res in responses:
                print(res)
            time.sleep(1 / 60)
        except grpc.RpcError as e:
            print(e.details())


run()



# クライアントから送信される映像データを表示する

# ============================================================
# import packages
# ============================================================
from concurrent import futures
import time
import cv2
import grpc
import base64
import numpy as np
import Server_pb2
import Server_pb2_grpc
import sys


class ShowVideoStream:
    img = None
    thread = futures.ThreadPoolExecutor(max_workers=1)

    # ==========
    def start(self):
        self.thread.submit(self.ShowWindow)

    # ==========
    def set(self, img):
        self.img = img

    # ==========
    def ShowWindow(self):
        while True:
            if self.img is not None:
                cv2.imshow('dst Image', self.img)
                k = cv2.waitKey(1)
                if k == 27:
                    break


class Greeter(Server_pb2_grpc.MainServerServicer):
    def __init__(self):
        pass

    def getStream(self, request_iterator, context):
        for req in request_iterator:
            dBuf = np.frombuffer(req.datas, dtype=np.uint8)
            dst = cv2.imdecode(dBuf, cv2.IMREAD_COLOR)
            yield Server_pb2.Reply(reply=1)


def serve():
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=10))
    Server_pb2_grpc.add_MainServerServicer_to_server(Greeter(), server)
    server.add_insecure_port('127.0.0.1:50051')
    server.start()
    print('server start')
    try:
        while True:
            time.sleep(1 / 60)
            # print("-")
    except KeyboardInterrupt:
        server.stop(0)


serve()



import os
from flask import Flask, request,render_template,url_for,redirect,send_file

import http.client
import json
from flask import abort

path = "/root/DXR_Project/DXR_FaceRec/face/database"


# 生成一个Flask对象
app = Flask(__name__)

# 路由
@app.route('/get_users', methods=['GET'])
def get_user():
    # 获取path目录下的所有文件名称进行返回
    file_list = os.listdir(path)
    # dit = {"users":file_list}
    # 遍历file_list，如果是文件夹，则添加到data中
    data = []
    for i in range(len(file_list)):
        if os.path.isdir(path+"/"+file_list[i]):
            print(file_list[i])
            # 过滤掉nohup.out,001,002,003
            if file_list[i] == "nohup.out" or file_list[i] == "001" or file_list[i] == "002" or file_list[i] == "003":
                continue
            data.append(file_list[i])
    print(data)
    return json.dumps(data)

@app.route('/get_user_list/<user_name>', methods=['GET'])
def get_user_list(user_name):
    # 获取path目录下的所有文件名称进行返回
    file_list = os.listdir(path+"/"+user_name)
    # 如果user_name是中文，则需要进行编码
    if user_name != user_name.encode('utf-8').decode('utf-8'):
        print("user_name is chinese")
        user_name = user_name.encode('utf-8').decode('utf-8')
    data = []
   # 存成一个数组，有两个字段，一个是name,一个是url
    for i in range(len(file_list)):
        data.append({"name":file_list[i],"url":f"/{user_name}/{file_list[i]}", "file_create_time":os.path.getctime(path+"/"+user_name+"/"+file_list[i])})
    return json.dumps(data)

# 添加用户
@app.route('/add_user', methods=['POST'])
def add_user():
    # 获取用户名称
    user_name = request.form['user_name']
    # user_name 是要创建的文件夹的绝对路径
    return_dit = {
        'code': 200,
        'msg': '添加成功'
    }
    print(user_name)
    # 判断user_name文件夹是否存在
    if os.path.exists(user_name):
        print("user_name is exists")
        return_dit['code'] = 400
        return_dit['msg'] = '用户已存在'
    else:
        os.mkdir(user_name)
    return json.dumps(return_dit)

# 删除用户
@app.route('/delete_user', methods=['POST'])
def delete_user():
    # 获取用户名称
    user_name = request.form['user_name']
    # user_name 是要创建的文件夹的绝对路径
    return_dit = {
        'code': 200,
        'msg': '删除成功'
    }
    # 判断user_name文件夹是否存在
    if os.path.exists(user_name):
        # 如果文件夹里面有文件，也要删除
        file_list = os.listdir(user_name)
        for i in range(len(file_list)):
            os.remove(user_name+"/"+file_list[i])
        os.rmdir(user_name)
    else:
        return_dit['code'] = 400
        return_dit['msg'] = '用户不存在'
    return json.dumps(return_dit)

# 上传图片
@app.route('/upload_image', methods=['POST'])
def upload_image():
    # 将图片存在/root/face_tmp下，并且返回上传图片的名称
    return_dit = {
        'code': 200,
        'msg': '上传成功',
        'data':{
            'image_name':''
        }
    }
    # 获取上传的文件
    file = request.files['file']
    # 获取上传的文件名
    file_name = file.filename
    print(file_name)
    # 获取上传的文件类型
    file_type = file_name.split('.')[-1]
    # 存储的文件,用原来的文件名
    file_path = '/root/face_tmp/'+file_name
    # 判断file_path是否存在,如果存在则把原来的文件备份
    if os.path.exists(file_path):
        os.rename(file_path,file_path+'.bak')
    # 保存文件
    file.save(file_path)
    # 返回文件名
    return_dit['data']['image_name'] = file_name
    return json.dumps(return_dit)

# 删除图片
@app.route('/delete_image', methods=['POST'])
def delete_image():
    # 删除图片,获取user_name,是要删除的文件的绝对路径
    user_name = request.form['user_name']
    print(user_name)
    return_dit = {
        'code': 200,
        'msg': '删除成功'
    }
    # 判断user_name文件夹是否存在
    if os.path.exists(user_name):
        os.remove(user_name)
    else:
        return_dit['code'] = 400
        return_dit['msg'] = '文件不存在'
    return json.dumps(return_dit)


@app.route('/download_file', methods=['GET'])
def download_file():
    # 获取文件的绝对路径
    file_path = request.args.get('remote_path')
    # 判断文件是否存在，如果存在则返回文件
    if os.path.exists(file_path):
        return send_file(file_path, as_attachment=True)
    else:
        # 如果文件不存在，则返回404
        return abort(404)
    

# 定义一个下载文件夹的方法
@app.route('/download_dir', methods=['GET'])
def download_dir():
    try:
        os.chdir('/root/nav_ws/src/lidar_localization/Localization/data')
        file_name = "latest"
        print(file_name)
        if os.path.exists(file_name + '.zip'):
            os.remove(file_name + '.zip')
        print(f'pwd: {os.getcwd()}')
        os.system('zip -r ' + file_name + '.zip ' + file_name)
        print('zip -r ' + file_name + '.zip ' + file_name)
        # 返回压缩后的文件
        return send_file(os.getcwd() + '/' + file_name + '.zip', as_attachment=True)
    except Exception as e:
        print(e)
        return abort(404)

@app.route('/upload_file', methods=['POST'])
def upload_file():
    try:
        remote_path = '/root/nav_ws/src/lidar_localization/Localization/data'
        os.chdir(remote_path)
        # 上传文件
        return_dit = {
            'code': 200,
            'msg': '上传成功'
        }
        # 获取上传的文件
        file = request.files['file']
        # 获取上传的文件名
        file_name = file.filename
        # 获取上传的文件类型
        file_type = file_name.split('.')[-1]
        # 保存文件
        # 判断文件是否存在，如果存在则删除
        if os.path.exists('./'+file_name):
            os.remove('./'+file_name)
        # 创建空文件
        os.system('touch '+'./'+file_name)
        file.save('./'+file_name)
        if file_type == 'zip':
            # 删掉原来的备份文件
            os.system('rm -rf '+file_name.split('.')[0]+'.bak')
            print(f'rm -rf {file_name.split(".")[0]}.bak')
            os.system('mv '+file_name.split('.')[0]+' '+file_name.split('.')[0]+'.bak')
            print(f'mv {file_name.split(".")[0]} {file_name.split(".")[0]}.bak')
            os.system('rm -rf '+file_name.split('.')[0])
            print(f'rm -rf {file_name.split(".")[0]}')
            # 查看当前目录
            print(f'pwd: {os.getcwd()}')
            # 查看当前目录下的文件
            print(f'ls: {os.listdir(os.getcwd())}')
            
            os.system('unzip '+file_name)
            print(f'unzip {file_name}')
            # 777权限
            os.system('chmod -R 777 '+file_name.split('.')[0])
            print(f'chmod -R 777 {file_name.split(".")[0]}')
        return json.dumps(return_dit)
    except Exception as e:
        print(e)
        return abort(404)
    
    

@app.route('/upload_single_file', methods=['POST'])
def upload_single_file():
    print('upload_single_file')
    try:
        remote_path = '/root/nav_ws/src/lidar_localization/Localization/data/latest'
        # 上传文件
        return_dit = {
            'code': 200,
            'msg': '上传成功'
        }
        # 获取上传的文件
        file = request.files['file']
        # 获取上传的文件名
        file_name = file.filename
        # 获取上传的文件类型
        file_type = file_name.split('.')[-1]
        print(f'file_name: {file_name} file_type: {file_type}')
        # 保存文件
        # 判断文件是否存在，如果存在，就把原来的文件备份
        if os.path.exists(remote_path+'/'+file_name):
            os.rename(remote_path+'/'+file_name,remote_path+'/'+file_name+'.bak')
        # 创建空文件
        os.system('touch '+remote_path+'/'+file_name)
        file.save(remote_path+'/'+file_name)
        # 777权限
        os.system('chmod -R 777 '+remote_path+'/'+file_name)
        return json.dumps(return_dit)
    except Exception as e:
        print(e)
        return abort(404)
            


if __name__ == '__main__':
    # 运行Flask
    app.run(host='0.0.0.0', port=5000)




import cv2
import subprocess as sp

class VideoStreamer:
    """
    VideoStreamer类用于创建和管理用于视频流传输的FFmpeg子进程。

    属性：
    srs_rtmp_url: RTMP URL，用于推送流到RTMP服务器。
    width: 视频宽度。
    height: 视频高度。
    command: FFmpeg子进程的命令行参数。
    pipe: FFmpeg子进程。

    方法：
    push_frame: 推送视频帧到RTMP服务器。
    close: 终止FFmpeg子进程。
    """
    
    def __init__(self, alias, ip="127.0.0.1", fps=30, width=1920, height=1080):
        # 将ip和别名拼接成RTMP URL
        self.srs_rtmp_url = f"rtmp://{ip}/live/{alias}"
        self.width = width
        self.height = height
        # 设置ffmpeg命令
        self.command = [
            'ffmpeg',
            '-y',
            '-f', 'rawvideo',
            '-vcodec', 'rawvideo',
            '-pix_fmt', 'bgr24',
            '-s', '{}x{}'.format(self.width, self.height),
            '-r', str(fps),
            '-i', '-',
            '-c:v', 'libx264',
            '-preset', 'superfast',
            '-tune', 'zerolatency',
            '-f', 'flv',
            self.srs_rtmp_url
        ]
        # 启动ffmpeg进程
        self.pipe = sp.Popen(self.command, stdin=sp.PIPE)

    def push_frame(self, frame):
        # 将帧推送到RTMP服务器
        self.pipe.stdin.write(frame.tobytes())

    def close(self):
        # 终止ffmpeg进程
        self.pipe.terminate()

if __name__ == "__main__":
    # 设置别名和视频流URL
    alias = "stream"
    pull_stream_url = 0

    # 打开视频流
    cap = cv2.VideoCapture(pull_stream_url)
    # 获取视频的宽度和高度
    width = int(cap.get(cv2.CAP_PROP_FRAME_WIDTH))
    height = int(cap.get(cv2.CAP_PROP_FRAME_HEIGHT))
    # 创建VideoStreamer对象
    streamer = VideoStreamer(alias, width=width, height=height)
    try:
        # 循环读取视频帧，并推送到RTMP服务器
        while True:
            ret, frame = cap.read()
            if not ret:
                break
            streamer.push_frame(frame)
    finally:
        # 释放资源
        cap.release()
        streamer.close()



from Dxr_grpc.dxr_grpc_server import Dxr_grpc_server
import cv2


def callbck(dst, json_data):
    # json_data的格式为：
    # {
    #     "type": "face  // 人脸识别",
    # }
    if json_data['type'] == 'face':
        # 模拟人脸识别
        cv2.rectangle(dst, (100, 100), (200, 200), (0, 0, 255), 2)
        cv2.putText(dst, 'face', (100, 100), cv2.FONT_HERSHEY_SIMPLEX, 1, (0, 0, 255), 2)   
    elif json_data['type'] == 'object':
        # 模拟物体识别
        cv2.rectangle(dst, (100, 100), (200, 200), (0, 255, 0), 2)
        cv2.putText(dst, 'object', (100, 100), cv2.FONT_HERSHEY_SIMPLEX, 1, (0, 255, 0), 2)
    # 返回处理后的图像和识别结果
    return dst, json_data

# 启动grpc服务，port为grpc服务端口，默认为50051，rec_callback为识别回调函数
server = Dxr_grpc_server(rec_callback=callbck)
server.run()


# クライアントから送信される映像データを表示する

# ============================================================
# import packages
# ============================================================
from concurrent import futures
import time
import cv2
import grpc
import base64
import numpy as np
import Server_pb2
import Server_pb2_grpc
import sys


class Greeter(Server_pb2_grpc.MainServerServicer):

    # ==========
    def __init__(self):
        pass

    # ==========
    def getStream(self, request_iterator, context):
        timer = 0

        # リクエストデータを表示クラスに渡す
        for req in request_iterator:
            print('process time = ' + str(time.clock() - timer))
            timer = time.clock()
            dBuf = np.frombuffer(req.datas, dtype=np.uint8)
            dst = cv2.imdecode(dBuf, cv2.IMREAD_COLOR)
            yield Server_pb2.Reply(reply=1)


def serve():
    # サーバーを生成
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=10))
    Server_pb2_grpc.add_MainServerServicer_to_server(Greeter(), server)
    # ポートを設定
    server.add_insecure_port('[::]:50051')
    # 動作開始
    server.start()
    print('server start')
    # プロセスが止まらないようにメインプロセスを常に動作させておく
    try:
        while True:
            time.sleep(1 / 60)
    except KeyboardInterrupt:
        server.stop(0)


serve()



import asyncio
import grpc
from grpc import aio
import zipfile
from . import zip_service_pb2
from . import zip_service_pb2_grpc
import logging

class ZipClient:
    def __init__(self, ip, zip_progress_callback, download_progress_callback):
        self.stub = None
        self.zip_progress_callback = zip_progress_callback
        self.download_progress_callback = download_progress_callback
        self.ip = ip

    async def start(self, directory):
        # Create gRPC channel and stub
        channel = aio.insecure_channel(f'{self.ip}:50051')
        self.stub = zip_service_pb2_grpc.ZipServiceStub(channel)

        # Call StartZip
        request = zip_service_pb2.ZipRequest(dir=directory)
        async for response in self.stub.StartZip(request):
            self.zip_progress_callback(response.progress)

        # Call DownloadZip
        downloaded_data = b''
        request = zip_service_pb2.DownloadRequest(filename='output.zip')
        async for response in self.stub.DownloadZip(request):
            if response.data:
                downloaded_data += response.data
                progress = response.sent_size / response.total_size
                self.download_progress_callback(progress)
                
        # Save the downloaded zip file
        with open('downloaded.zip', 'wb') as f:
            f.write(downloaded_data)

        await channel.close()

    def start_sync(self, directory):
        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)
        try:
            loop.run_until_complete(self.start(directory))
        finally:
            loop.close()
            asyncio.set_event_loop(None)


def zip_progress_update(progress):
    print('Zip Progress: {:.2f} %'.format(progress * 100))

def download_progress_update(progress):
    print('Download Progress: {:.2f} %'.format(progress * 100))


# User code:
if __name__ == '__main__':
    logging.basicConfig()
    client = ZipClient(zip_progress_update, download_progress_update)
    client.start_sync('/Users/luzhipeng/Desktop/Images/图片')
